"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var pg_exports = {};
__export(pg_exports, {
  generateSchemaData: () => generateSchemaData
});
module.exports = __toCommonJS(pg_exports);
var import_drizzle_orm = require("drizzle-orm");
var import_pg_core = require("drizzle-orm/pg-core");
var import_graphql = require("graphql");
var import_common = require("./common");
var import_caseOps = require("../../caseOps");
var import_dataMappers = require("../../dataMappers");
const generateSelectArray = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${(0, import_caseOps.camelize)(tableName)}`;
  const queryBase = db.query[tableName];
  if (!queryBase)
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  const queryArgs = {
    offset: {
      type: import_graphql.GraphQLInt
    },
    limit: {
      type: import_graphql.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { offset, limit, orderBy, where } = args;
      const tableSelection = info.operation.selectionSet.selections.find(
        (e) => e.kind === import_graphql.Kind.FIELD && e.name.value === queryName
      );
      const query = queryBase.findMany({
        columns: (0, import_common.extractSelectedColumnsFromNode)(tableSelection, table),
        offset,
        limit,
        orderBy: orderBy ? (0, import_common.extractOrderBy)(table, orderBy) : void 0,
        where: where ? (0, import_common.extractFilters)(table, tableName, where) : void 0,
        with: relations ? (0, import_common.extractRelationsParams)(relations, tableSelection, `${(0, import_caseOps.pascalize)(tableName)}SelectItem`, info) : void 0
      });
      const result = await query;
      return (0, import_dataMappers.remapToGraphQLArrayOutput)(result, relations);
    },
    args: queryArgs
  };
};
const generateSelectSingle = (db, tableName, table, relations, orderArgs, filterArgs) => {
  const queryName = `${(0, import_caseOps.camelize)(tableName)}Single`;
  const queryBase = db.query[tableName];
  if (!queryBase)
    throw new Error(
      `Table ${tableName} not found in drizzle instance. Did you forget to pass schema to drizzle constructor?`
    );
  const queryArgs = {
    offset: {
      type: import_graphql.GraphQLInt
    },
    limit: {
      type: import_graphql.GraphQLInt
    },
    orderBy: {
      type: orderArgs
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { offset, orderBy, where } = args;
      const tableSelection = info.operation.selectionSet.selections.find(
        (e) => e.kind === import_graphql.Kind.FIELD && e.name.value === queryName
      );
      const query = queryBase.findFirst({
        columns: (0, import_common.extractSelectedColumnsFromNode)(tableSelection, table),
        offset,
        orderBy: orderBy ? (0, import_common.extractOrderBy)(table, orderBy) : void 0,
        where: where ? (0, import_common.extractFilters)(table, tableName, where) : void 0,
        with: relations ? (0, import_common.extractRelationsParams)(relations, tableSelection, `${(0, import_caseOps.pascalize)(tableName)}SelectItem`, info) : void 0
      });
      const result = await query;
      if (!result)
        return void 0;
      return (0, import_dataMappers.remapToGraphQLSingleOutput)(result, relations);
    },
    args: queryArgs
  };
};
const generateInsertArray = (db, tableName, table, baseType) => {
  const queryName = `insertInto${(0, import_caseOps.pascalize)(tableName)}`;
  const queryArgs = {
    values: {
      type: new import_graphql.GraphQLNonNull(new import_graphql.GraphQLList(new import_graphql.GraphQLNonNull(baseType)))
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const input = (0, import_dataMappers.remapFromGraphQLArrayInput)(args.values, table);
      if (!input.length)
        throw new import_graphql.GraphQLError("No values were provided!");
      const columns = (0, import_common.extractSelectedColumnsSQLFormat)(info, queryName, table);
      const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
      return (0, import_dataMappers.remapToGraphQLArrayOutput)(result);
    },
    args: queryArgs
  };
};
const generateInsertSingle = (db, tableName, table, baseType) => {
  const queryName = `insertInto${(0, import_caseOps.pascalize)(tableName)}Single`;
  const queryArgs = {
    values: {
      type: new import_graphql.GraphQLNonNull(baseType)
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const input = (0, import_dataMappers.remapFromGraphQLSingleInput)(args.values, table);
      const columns = (0, import_common.extractSelectedColumnsSQLFormat)(info, queryName, table);
      const result = await db.insert(table).values(input).returning(columns).onConflictDoNothing();
      if (!result[0])
        return void 0;
      return (0, import_dataMappers.remapToGraphQLSingleOutput)(result[0]);
    },
    args: queryArgs
  };
};
const generateUpdate = (db, tableName, table, setArgs, filterArgs) => {
  const queryName = `update${(0, import_caseOps.pascalize)(tableName)}`;
  const queryArgs = {
    set: {
      type: new import_graphql.GraphQLNonNull(setArgs)
    },
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { where, set } = args;
      const columns = (0, import_common.extractSelectedColumnsSQLFormat)(info, queryName, table);
      const input = (0, import_dataMappers.remapFromGraphQLSingleInput)(set, table);
      if (!Object.keys(input).length)
        throw new import_graphql.GraphQLError("Unable to update with no values specified!");
      let query = db.update(table).set(input);
      if (where) {
        const filters = (0, import_common.extractFilters)(table, tableName, where);
        query = query.where(filters);
      }
      query = query.returning(columns);
      const result = await query;
      return (0, import_dataMappers.remapToGraphQLArrayOutput)(result);
    },
    args: queryArgs
  };
};
const generateDelete = (db, tableName, table, filterArgs) => {
  const queryName = `deleteFrom${(0, import_caseOps.pascalize)(tableName)}`;
  const queryArgs = {
    where: {
      type: filterArgs
    }
  };
  return {
    name: queryName,
    resolver: async (source, args, context, info) => {
      const { where } = args;
      const columns = (0, import_common.extractSelectedColumnsSQLFormat)(info, queryName, table);
      let query = db.delete(table);
      if (where) {
        const filters = (0, import_common.extractFilters)(table, tableName, where);
        query = query.where(filters);
      }
      query = query.returning(columns);
      const result = await query;
      return (0, import_dataMappers.remapToGraphQLArrayOutput)(result);
    },
    args: queryArgs
  };
};
const generateSchemaData = (db, schema) => {
  const rawSchema = schema;
  const schemaEntries = Object.entries(rawSchema);
  const tables = Object.fromEntries(schemaEntries.filter(([key, value]) => (0, import_drizzle_orm.is)(value, import_pg_core.PgTable)));
  if (!tables || !Object.keys(tables).length)
    throw new Error(
      `Unable to extract tables from drizzle instance.
Did you forget to pass tables to graphql schema constructor?`
    );
  const relations = Object.fromEntries(
    schemaEntries.filter(([key, value]) => (0, import_drizzle_orm.is)(value, import_drizzle_orm.Relations)).map(([key, value]) => [
      Object.entries(tables).find(
        ([tableName, tableValue]) => tableValue === value.table
      )[0],
      value
    ]).map(([tableName, relValue]) => [
      tableName,
      relValue.config((0, import_drizzle_orm.createTableRelationsHelpers)(tables[tableName]))
    ])
  );
  const queries = {};
  const mutations = {};
  const gqlSchemaTypes = Object.fromEntries(
    Object.entries(tables).map(([tableName, table]) => [
      tableName,
      (0, import_common.generateTableTypes)(tableName, table, true, relations[tableName])
    ])
  );
  const inputs = {};
  const outputs = {};
  for (const [tableName, tableTypes] of Object.entries(gqlSchemaTypes)) {
    const { insertInput, updateInput, tableFilters, tableOrder } = tableTypes.inputs;
    const { selectSingleOutput, selectArrOutput, singleTableItemOutput, arrTableItemOutput } = tableTypes.outputs;
    const selectArrGenerated = generateSelectArray(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const selectSingleGenerated = generateSelectSingle(
      db,
      tableName,
      schema[tableName],
      relations[tableName],
      tableOrder,
      tableFilters
    );
    const insertArrGenerated = generateInsertArray(db, tableName, schema[tableName], insertInput);
    const insertSingleGenerated = generateInsertSingle(db, tableName, schema[tableName], insertInput);
    const updateGenerated = generateUpdate(db, tableName, schema[tableName], updateInput, tableFilters);
    const deleteGenerated = generateDelete(db, tableName, schema[tableName], tableFilters);
    queries[selectArrGenerated.name] = {
      type: selectArrOutput,
      args: selectArrGenerated.args,
      resolve: selectArrGenerated.resolver
    };
    queries[selectSingleGenerated.name] = {
      type: selectSingleOutput,
      args: selectSingleGenerated.args,
      resolve: selectSingleGenerated.resolver
    };
    mutations[insertArrGenerated.name] = {
      type: arrTableItemOutput,
      args: insertArrGenerated.args,
      resolve: insertArrGenerated.resolver
    };
    mutations[insertSingleGenerated.name] = {
      type: singleTableItemOutput,
      args: insertSingleGenerated.args,
      resolve: insertSingleGenerated.resolver
    };
    mutations[updateGenerated.name] = {
      type: arrTableItemOutput,
      args: updateGenerated.args,
      resolve: updateGenerated.resolver
    };
    mutations[deleteGenerated.name] = {
      type: arrTableItemOutput,
      args: deleteGenerated.args,
      resolve: deleteGenerated.resolver
    };
    [insertInput, updateInput, tableFilters, tableOrder].forEach((e) => inputs[e.name] = e);
    outputs[selectSingleOutput.name] = selectSingleOutput;
    outputs[singleTableItemOutput.name] = singleTableItemOutput;
  }
  return { queries, mutations, inputs, types: outputs };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  generateSchemaData
});
//# sourceMappingURL=pg.cjs.map